VADS_ACTION_MODE_CHOICES = (
    ('INTERACTIVE', 'INTERACTIVE'),
    ('SILENT', 'SILENT'),
)

VADS_CURRENCY_CHOICES = (
    ('036', 'Australian dollar'),
    ('124', 'Canadian dollar'),
    ('156', 'Chinese Yuan'),
    ('208', 'Danish Krone'),
    ('392', 'Japanese Yen'),
    ('578', 'Norwegian Krone'),
    ('752', 'Swedish Krona'),
    ('756', 'Swiss franc'),
    ('826', 'Pound sterling'),
    ('840', 'American dollar'),
    ('953', 'Franc Pacifique (CFP)'),
    ('978', 'Euro')
)

VADS_CTX_MODE_CHOICES = (
    ('TEST', 'TEST'),
    ('PRODUCTION', 'PRODUCTION')
)

VADS_RETURN_MODE_CHOICES = (
    ('AMEX', 'American Express'),
    ('AURORE-MULTI', 'AURORE (multi brand)'),
    ('BUYSTER', 'BUYSTER'),
    ('CB', 'CB'),
    ('COFINOGA', 'COFINOGA'),
    ('E-CARTEBLEUE', 'e blue card'),
    ('MASTERCARD', 'Eurocard / MasterCard'),
    ('JCB', 'JCB'),
    ('MAESTRO', 'Maestro'),
    ('ONEY', 'ONEY'),
    ('ONEY_SANDBOX', 'ONEY SANDBOX mode'),
    ('PAYPAL', 'PAYPAL'),
    ('PAYPAL_SB', 'PAYPAL SANDBOX mode'),
    ('PAYSAFECARD', 'PAYSAFECARD'),
    ('VISA', 'Visa'),
    ('VISA_ELECTRON', 'Visa Electron'),
    ('COF3XCB', '3x CB Cofinoga'),
    ('COF3XCB_SB', '3x CB Cofinoga SANDBOX'),
)

VADS_VALIDATION_MODE_CHOICES = (
    ('', 'Default shop configuration (using payzen admin)'),
    ('0', 'Automatic validation'),
    ('1', 'Manual validation')
)

VADS_TRANS_STATUS = (
    ('ABANDONED', 'ABANDONED'),
    ('AUTHORISED', 'AUTHORISED'),
    ('REFUSED', 'REFUSED'),
    ('AUTHORISED_TO_VALIDATE', 'AUTHORISED_TO_VALIDATE'),
    ('WAITING_AUTHORISATION', 'WAITING_AUTHORISATION'),
    ('EXPIRED', 'EXPIRED'),
    ('CANCELLED', 'CANCELLED'),
    ('WAITING_AUTHORISATION_TO_VALIDATE', 'WAITING_AUTHORISATION_TO_VALIDATE'),
    ('CAPTURED', 'CAPTURED'),
)

VADS_OPERATION_TYPE_CHOICES = (
    ('DEBIT', 'DEBIT'),
    ('CREDIT', 'CREDIT')
)

VADS_RESULT_CHOICES = (
    ('00', 'Payment successful'),
    ('02', 'Merchant should contact his bank'),
    ('05', 'Payment refused'),
    ('17', 'Payment cancelled by client'),
    ('30', 'Wrong request format'),
    ('96', 'Technical error during payment process')
)

VADS_AUTH_MODE_CHOICES = (
    ('FULL', 'FULL'),
    ('MARK', 'MARK')
)

VADS_THREEDS_ENROLLED = (
    ('Y', 'Y'),
    ('N', 'N'),
    ('U', 'U')
)

VADS_THREEDS_CAVVALGORITHM_CHOICES = (
    ('0', 'HMAC'),
    ('1', 'CVV'),
    ('2', 'CVV_ATN'),
    ('3', 'Mastercard SPA')
)

VADS_THREEDS_STATUS_CHOICES = (
    ('Y', 'U'),
    ('N', 'N'),
    ('U', 'U'),
    ('A', 'A')
)

VADS_URL_CHECK_SRC_CHOICES = (
    ('PAY', 'PAY'),
    ('BO', 'BO'),
    ('BATCH', 'BATCH'),
    ('BATCH_AUTO', 'BATCH_AUTO'),
    ('FILE', 'FILE'),
    ('REC', 'REC'),
    ('MERCH_BO', 'MERCH_BO')
)

VADS_PAYMENT_SRC_CHOICES = (
    ('EC', 'EC'),
    ('MOTO', 'MOTO'),
    ('CC', 'CC'),
    ('OTHER', 'OTHER')
)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('django_payzen', '0003_auto_20141027_1647'),
    ]

    operations = [
        migrations.AlterField(
            model_name='paymentrequest',
            name='vads_contrib',
            field=models.CharField(default=b'django-payzen v1.0', max_length=255, null=True, blank=True),
            preserve_default=True,
        ),
    ]



""" Default payzen settings.

All settings should be set in the django settings file and not
directly in this file.
We deliberately do not set up default values here in order to
force user to setup explicitely the default behaviour."""

from django.conf import settings


PAYZEN_REQUEST_URL = 'https://secure.payzen.eu/vads-payment/'
VADS_CONTRIB = 'django-payzen v1.0'
VADS_SITE_ID = getattr(settings, 'VADS_SITE_ID')
VADS_CERTIFICATE = getattr(settings, 'VADS_CERTIFICATE')
VADS_CTX_MODE = getattr(settings, 'VADS_CTX_MODE')
VADS_CURRENCY = getattr(settings, 'VADS_CURRENCY', None)
VADS_ACTION_MODE = getattr(settings, 'VADS_ACTION_MODE', None)



from django import template

from .. import app_settings
from .. import forms

register = template.base.Library()


@register.simple_tag
def payzen_form(payment_request, auto_submit=False):
    """TODO docstring."""
    if auto_submit:
        template_used = "django_payzen/auto_submit_form.html"
    else:
        template_used = "django_payzen/form.html"
    payment_request.update()
    t = template.loader.get_template(template_used)
    return t.render(template.Context({
        "form": forms.PaymentRequestForm(instance=payment_request),
        "payzen_submit_url": app_settings.PAYZEN_REQUEST_URL
    }))



import datetime

from django.test import TestCase
from django import template

from .. import data
from ... import models


class RequestDataTester(object):

    def analyze_template_tag_response(self, response):
        # Test that nothing is added before template tag
        self.assertTrue(
            response.find('<form action=' +
                          '"https://secure.payzen.eu/vads-payment/"' +
                          ' method="post">') == 0)
        # Test that nothing is added after template tag
        self.assertEqual(
            response[-8:], u'</form>\n')
        # test that all fields are present
        self.assertEqual(
            response.count("input"),
            self.expected_values['fields_nb'])

    def test_payzen_form_template_tag(self):
        t = template.Template(
            "{% load payzen_extras %}{% payzen_form instance %}")
        c = template.Context({
            "instance": self.instance
        })
        response = t.render(c)
        self.analyze_template_tag_response(response)

    def test_vads_theme_config_value(self):
        self.assertEqual(
            self.expected_values['vads_theme_config'],
            self.instance.vads_theme_config)

    def test_vads_payment_config_value(self):
        self.assertEqual(
            self.expected_values['vads_payment_config'],
            self.instance.vads_payment_config)

    def test_signature_value(self):
        self.assertTrue(
            self.instance.signature and len(self.instance.signature) == 40
        )


class BasicPaymentTest(RequestDataTester, TestCase):

    def setUp(self):
        self.instance = models.PaymentRequest(
            **data.payment_args)
        self.instance.save()
        self.expected_values = {
            "fields_nb": 56,
            "vads_theme_config": None,
            "vads_payment_config": "SINGLE",
            "vads_trans_id": "000001"
        }


class CustomizedPaymentTest(RequestDataTester, TestCase):

    def setUp(self):
        self.instance = models.PaymentRequest(
            **data.payment_args)
        self.instance.payment_config = models.MultiPaymentConfig(
            **data.payment_config_args)
        self.instance.payment_config.save()
        self.instance.save()

        self.expected_values = {
            "fields_nb": 57,
            "vads_theme_config": None,
            "vads_payment_config": "MULTI:first=5000;count=2;period=5",
            "vads_trans_id": "000001"
        }


class CustomizedThemeTest(RequestDataTester, TestCase):

    def setUp(self):
        self.instance = models.PaymentRequest(
            **data.payment_args)
        self.instance.theme = models.ThemeConfig(
            **data.theme_args)
        self.instance.theme.save()
        self.instance.save()

        self.expected_values = {
            "fields_nb": 58,
            "vads_theme_config": self.get_theme_string(),
            "vads_payment_config": "SINGLE",
            "vads_trans_id": "000001"
        }

    def get_theme_string(self):
        theme_string = ""
        for key, value in data.theme_args.items():
            theme_string += "{}={};".format(
                key.upper(), value)
        return theme_string[:-1]


class MultiPaymentConfigTest(RequestDataTester, TestCase):

    def setUp(self):
        self.instance = models.PaymentRequest(
            **data.payment_args)
        self.instance.save()
        for i in range(1, 6):
            cpc = models.CustomPaymentConfig(
                date=datetime.date(2015, i, 1),
                amount=200)
            cpc.save()
            self.instance.custom_payment_config.add(cpc)
        self.instance.update()

        self.expected_values = {
            "fields_nb": 56,
            "vads_theme_config": None,
            "vads_payment_config": "MULTI_EXT:20150101=200;" +
                                   "20150201=200;" +
                                   "20150301=200;" +
                                   "20150401=200;" +
                                   "20150501=200",
            "vads_trans_id": "000001"
        }



import logging

from django import http
from django.views import generic
from django.views.decorators.csrf import csrf_exempt

from . import forms
from . import models
from . import signals
from . import tools

logger = logging.getLogger(__name__)


class ResponseView(generic.View):

    http_method_names = [u'post']

    @csrf_exempt
    def dispatch(self, request, *args, **kwargs):
        # We check if the signature is valid. If not return
        if not tools.is_signature_valid(request.POST):
            logger.warning(
                "Django-Payzen : Response signature detected as invalid",
                extra={"stack": True})
            return http.HttpResponse()
        # The signature is valid
        vads_trans_id = request.POST.get("vads_trans_id")
        vads_trans_date = request.POST.get("vads_trans_date")
        vads_site_id = request.POST.get("vads_site_id")
        try:
            instance = models.PaymentResponse.objects.get(
                vads_trans_id=vads_trans_id,
                vads_trans_date=vads_trans_date,
                vads_site_id=vads_site_id)
            form = forms.PaymentResponseForm(request.POST, instance=instance)
        except models.PaymentResponse.DoesNotExist:
            form = forms.PaymentResponseForm(request.POST)
        if form.is_valid():
            response = form.save()
            if response.payment_successful:
                signal = signals.payment_success
            else:
                signal = signals.payment_failure
            signal.send(sender=self.__class__, response=response)
            logger.info("Django-Payzen : Transaction {} response received !"
                        .format(response.vads_trans_id))
        else:
            signals.response_error.send(sender=self.__class__)
            logger.error("Django-Payzen : Response could not be saved - {} {}"
                         .format(form.errors, request.POST),
                         extra={"stack": True})
        return http.HttpResponse()



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='CustomPaymentConfig',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('date', models.DateField()),
                ('amount', models.PositiveIntegerField()),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='MultiPaymentConfig',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('first', models.PositiveIntegerField()),
                ('count', models.PositiveIntegerField()),
                ('period', models.PositiveIntegerField()),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='PaymentRequest',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('vads_cust_address', models.CharField(max_length=255, null=True, blank=True)),
                ('vads_cust_address_number', models.CharField(max_length=5, null=True, blank=True)),
                ('vads_cust_country', models.CharField(max_length=2, null=True, blank=True)),
                ('vads_cust_email', models.EmailField(max_length=75, null=True, blank=True)),
                ('vads_cust_id', models.CharField(max_length=63, null=True, blank=True)),
                ('vads_cust_name', models.CharField(max_length=127, null=True, blank=True)),
                ('vads_cust_last_name', models.CharField(max_length=63, null=True, blank=True)),
                ('vads_cust_first_name', models.CharField(max_length=63, null=True, blank=True)),
                ('vads_cust_cell_phone', models.CharField(max_length=32, null=True, blank=True)),
                ('vads_cust_phone', models.CharField(max_length=32, null=True, blank=True)),
                ('vads_cust_title', models.CharField(max_length=63, null=True, blank=True)),
                ('vads_cust_city', models.CharField(max_length=63, null=True, blank=True)),
                ('vads_cust_status', models.CharField(max_length=63, null=True, blank=True)),
                ('vads_cust_state', models.CharField(max_length=63, null=True, blank=True)),
                ('vads_cust_zip', models.CharField(max_length=63, null=True, blank=True)),
                ('vads_language', models.CharField(max_length=2, null=True, blank=True)),
                ('vads_order_id', models.CharField(max_length=32, null=True, blank=True)),
                ('vads_order_info', models.CharField(max_length=255, null=True, blank=True)),
                ('vads_order_info2', models.CharField(max_length=255, null=True, blank=True)),
                ('vads_order_info3', models.CharField(max_length=255, null=True, blank=True)),
                ('vads_ship_to_name', models.CharField(max_length=127, null=True, blank=True)),
                ('vads_ship_to_first_name', models.CharField(max_length=63, null=True, blank=True)),
                ('vads_ship_to_last_name', models.CharField(max_length=63, null=True, blank=True)),
                ('vads_ship_to_street_number', models.CharField(max_length=5, null=True, blank=True)),
                ('vads_ship_to_street', models.CharField(max_length=255, null=True, blank=True)),
                ('vads_ship_to_street2', models.CharField(max_length=255, null=True, blank=True)),
                ('vads_ship_to_zip', models.CharField(max_length=63, null=True, blank=True)),
                ('vads_ship_to_city', models.CharField(max_length=63, null=True, blank=True)),
                ('vads_ship_to_country', models.CharField(max_length=2, null=True, blank=True)),
                ('vads_ship_to_phone_num', models.CharField(max_length=32, null=True, blank=True)),
                ('vads_ship_to_state', models.CharField(max_length=255, null=True, blank=True)),
                ('vads_action_mode', models.CharField(default=b'INTERACTIVE', max_length=11, choices=[(b'INTERACTIVE', b'INTERACTIVE'), (b'SILENT', b'SILENT')])),
                ('vads_amount', models.PositiveIntegerField()),
                ('vads_currency', models.CharField(default=b'978', max_length=3, choices=[(b'036', b'Australian dollar'), (b'124', b'Canadian dollar'), (b'156', b'Chinese Yuan'), (b'208', b'Danish Krone'), (b'392', b'Japanese Yen'), (b'578', b'Norwegian Krone'), (b'752', b'Swedish Krona'), (b'756', b'Swiss franc'), (b'826', b'Pound sterling'), (b'840', b'American dollar'), (b'953', b'Franc Pacifique (CFP)'), (b'978', b'Euro')])),
                ('vads_ctx_mode', models.CharField(default=b'TEST', max_length=10, choices=[(b'TEST', b'TEST'), (b'PRODUCTION', b'PRODUCTION')])),
                ('vads_page_action', models.CharField(default=b'PAYMENT', max_length=7)),
                ('vads_payment_config', models.TextField()),
                ('vads_site_id', models.PositiveIntegerField(default=b'29822824')),
                ('vads_trans_date', models.CharField(max_length=14)),
                ('vads_trans_id', models.CharField(max_length=6)),
                ('vads_version', models.CharField(default=b'V2', max_length=2)),
                ('signature', models.CharField(max_length=40)),
                ('vads_capture_delay', models.PositiveIntegerField(null=True, blank=True)),
                ('vads_contrib', models.CharField(default=b'django-payzen v0.9', max_length=255, null=True, blank=True)),
                ('vads_payment_cards', models.CharField(max_length=127, null=True, blank=True)),
                ('vads_return_mode', models.CharField(blank=True, max_length=12, null=True, choices=[(b'AMEX', b'American Express'), (b'AURORE-MULTI', b'AURORE (multi brand)'), (b'BUYSTER', b'BUYSTER'), (b'CB', b'CB'), (b'COFINOGA', b'COFINOGA'), (b'E-CARTEBLEUE', b'e blue card'), (b'MASTERCARD', b'Eurocard / MasterCard'), (b'JCB', b'JCB'), (b'MAESTRO', b'Maestro'), (b'ONEY', b'ONEY'), (b'ONEY_SANDBOX', b'ONEY SANDBOX mode'), (b'PAYPAL', b'PAYPAL'), (b'PAYPAL_SB', b'PAYPAL SANDBOX mode'), (b'PAYSAFECARD', b'PAYSAFECARD'), (b'VISA', b'Visa'), (b'VISA_ELECTRON', b'Visa Electron'), (b'COF3XCB', b'3x CB Cofinoga'), (b'COF3XCB_SB', b'3x CB Cofinoga SANDBOX')])),
                ('vads_theme_config', models.CharField(max_length=255, null=True, blank=True)),
                ('vads_validation_mode', models.CharField(blank=True, max_length=1, null=True, choices=[(b'', b'Default shop configuration (using payzen admin)'), (b'0', b'Automatic validation'), (b'1', b'Manual validation')])),
                ('vads_url_success', models.URLField(null=True, blank=True)),
                ('vads_url_referral', models.URLField(null=True, blank=True)),
                ('vads_url_refused', models.URLField(null=True, blank=True)),
                ('vads_url_cancel', models.URLField(null=True, blank=True)),
                ('vads_url_error', models.URLField(null=True, blank=True)),
                ('vads_url_return', models.URLField(null=True, blank=True)),
                ('vads_user_info', models.CharField(max_length=255, null=True, blank=True)),
                ('vads_shop_name', models.CharField(max_length=255, null=True, blank=True)),
                ('vads_redirect_success_timeout', models.PositiveIntegerField(null=True, blank=True)),
                ('vads_redirect_success_message', models.CharField(max_length=255, null=True, blank=True)),
                ('vads_redirect_error_timeout', models.PositiveIntegerField(null=True, blank=True)),
                ('vads_redirect_error_message', models.CharField(max_length=255, null=True, blank=True)),
                ('custom_payment_config', models.ManyToManyField(to='django_payzen.CustomPaymentConfig')),
                ('payment_config', models.ForeignKey(blank=True, to='django_payzen.MultiPaymentConfig', null=True)),
            ],
            options={
                'verbose_name': 'Request',
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='PaymentResponse',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('vads_cust_address', models.CharField(max_length=255, null=True, blank=True)),
                ('vads_cust_address_number', models.CharField(max_length=5, null=True, blank=True)),
                ('vads_cust_country', models.CharField(max_length=2, null=True, blank=True)),
                ('vads_cust_email', models.EmailField(max_length=75, null=True, blank=True)),
                ('vads_cust_id', models.CharField(max_length=63, null=True, blank=True)),
                ('vads_cust_name', models.CharField(max_length=127, null=True, blank=True)),
                ('vads_cust_last_name', models.CharField(max_length=63, null=True, blank=True)),
                ('vads_cust_first_name', models.CharField(max_length=63, null=True, blank=True)),
                ('vads_cust_cell_phone', models.CharField(max_length=32, null=True, blank=True)),
                ('vads_cust_phone', models.CharField(max_length=32, null=True, blank=True)),
                ('vads_cust_title', models.CharField(max_length=63, null=True, blank=True)),
                ('vads_cust_city', models.CharField(max_length=63, null=True, blank=True)),
                ('vads_cust_status', models.CharField(max_length=63, null=True, blank=True)),
                ('vads_cust_state', models.CharField(max_length=63, null=True, blank=True)),
                ('vads_cust_zip', models.CharField(max_length=63, null=True, blank=True)),
                ('vads_language', models.CharField(max_length=2, null=True, blank=True)),
                ('vads_order_id', models.CharField(max_length=32, null=True, blank=True)),
                ('vads_order_info', models.CharField(max_length=255, null=True, blank=True)),
                ('vads_order_info2', models.CharField(max_length=255, null=True, blank=True)),
                ('vads_order_info3', models.CharField(max_length=255, null=True, blank=True)),
                ('vads_ship_to_name', models.CharField(max_length=127, null=True, blank=True)),
                ('vads_ship_to_first_name', models.CharField(max_length=63, null=True, blank=True)),
                ('vads_ship_to_last_name', models.CharField(max_length=63, null=True, blank=True)),
                ('vads_ship_to_street_number', models.CharField(max_length=5, null=True, blank=True)),
                ('vads_ship_to_street', models.CharField(max_length=255, null=True, blank=True)),
                ('vads_ship_to_street2', models.CharField(max_length=255, null=True, blank=True)),
                ('vads_ship_to_zip', models.CharField(max_length=63, null=True, blank=True)),
                ('vads_ship_to_city', models.CharField(max_length=63, null=True, blank=True)),
                ('vads_ship_to_country', models.CharField(max_length=2, null=True, blank=True)),
                ('vads_ship_to_phone_num', models.CharField(max_length=32, null=True, blank=True)),
                ('vads_ship_to_state', models.CharField(max_length=255, null=True, blank=True)),
                ('vads_threeds_enrolled', models.CharField(blank=True, max_length=1, null=True, choices=[(b'Y', b'Y'), (b'N', b'N'), (b'U', b'U')])),
                ('vads_threeds_cavv', models.CharField(max_length=28, null=True, blank=True)),
                ('vads_threeds_eci', models.CharField(max_length=2, null=True, blank=True)),
                ('vads_threeds_xid', models.CharField(max_length=28, null=True, blank=True)),
                ('vads_threeds_cavvAlgorithm', models.CharField(blank=True, max_length=1, null=True, choices=[(b'0', b'HMAC'), (b'1', b'CVV'), (b'2', b'CVV_ATN'), (b'3', b'Mastercard SPA')])),
                ('vads_threeds_status', models.CharField(blank=True, max_length=1, null=True, choices=[(b'Y', b'U'), (b'N', b'N'), (b'U', b'U'), (b'A', b'A')])),
                ('vads_threeds_sign_valid', models.NullBooleanField()),
                ('vads_threeds_error_code', models.CharField(max_length=2, null=True, blank=True)),
                ('vads_threeds_exit_status', models.CharField(max_length=2, null=True, blank=True)),
                ('vads_warranty_result', models.CharField(max_length=13, null=True, blank=True)),
                ('vads_available_languages', models.TextField(null=True, blank=True)),
                ('vads_theme_config', models.CharField(max_length=255, null=True, blank=True)),
                ('vads_shop_url', models.URLField(null=True, blank=True)),
                ('vads_shop_name', models.CharField(max_length=255, null=True, blank=True)),
                ('signature', models.CharField(max_length=40)),
                ('vads_ctx_mode', models.CharField(max_length=10, choices=[(b'TEST', b'TEST'), (b'PRODUCTION', b'PRODUCTION')])),
                ('vads_url_check_src', models.CharField(max_length=10, choices=[(b'PAY', b'PAY'), (b'BO', b'BO'), (b'BATCH_AUTO', b'BATCH_AUTO'), (b'FILE', b'FILE'), (b'REC', b'REC'), (b'MERCH_BO', b'MERCH_BO')])),
                ('vads_version', models.CharField(max_length=2)),
                ('vads_trans_date', models.CharField(max_length=14)),
                ('vads_action_mode', models.CharField(max_length=11, choices=[(b'INTERACTIVE', b'INTERACTIVE'), (b'SILENT', b'SILENT')])),
                ('vads_trans_id', models.CharField(max_length=6)),
                ('vads_payment_config', models.TextField(null=True, blank=True)),
                ('vads_sequence_number', models.PositiveSmallIntegerField(null=True, blank=True)),
                ('vads_site_id', models.PositiveIntegerField()),
                ('vads_amount', models.PositiveIntegerField(null=True, blank=True)),
                ('vads_currency', models.CharField(blank=True, max_length=3, null=True, choices=[(b'036', b'Australian dollar'), (b'124', b'Canadian dollar'), (b'156', b'Chinese Yuan'), (b'208', b'Danish Krone'), (b'392', b'Japanese Yen'), (b'578', b'Norwegian Krone'), (b'752', b'Swedish Krona'), (b'756', b'Swiss franc'), (b'826', b'Pound sterling'), (b'840', b'American dollar'), (b'953', b'Franc Pacifique (CFP)'), (b'978', b'Euro')])),
                ('vads_effective_amount', models.PositiveIntegerField(null=True, blank=True)),
                ('vads_operation_type', models.CharField(blank=True, max_length=6, null=True, choices=[(b'DEBIT', b'DEBIT'), (b'CREDIT', b'CREDIT')])),
                ('vads_result', models.CharField(max_length=2, choices=[(b'00', b'Payment successful'), (b'02', b'Merchant should contact his bank'), (b'05', b'Payment refused'), (b'30', b'Wrong request format'), (b'96', b'Technical error during payment process')])),
                ('vads_validation_mode', models.CharField(blank=True, max_length=1, null=True, choices=[(b'', b'Default shop configuration (using payzen admin)'), (b'0', b'Automatic validation'), (b'1', b'Manual validation')])),
                ('vads_trans_status', models.CharField(max_length=33, choices=[(b'ABANDONED', b'ABANDONED'), (b'AUTHORISED', b'AUTHORISED'), (b'REFUSED', b'REFUSED'), (b'AUTHORISED_TO_VALIDATE', b'AUTHORISED_TO_VALIDATE'), (b'WAITING_AUTHORISATION', b'WAITING_AUTHORISATION'), (b'EXPIRED', b'EXPIRED'), (b'CANCELLED', b'CANCELLED'), (b'WAITING_AUTHORISATION_TO_VALIDATE', b'WAITING_AUTHORISATION_TO_VALIDATE'), (b'CAPTURED', b'CAPTURED')])),
                ('vads_effective_creation_date', models.CharField(max_length=14, null=True, blank=True)),
                ('vads_presentation_date', models.CharField(max_length=14, null=True, blank=True)),
                ('vads_capture_delay', models.PositiveIntegerField(null=True, blank=True)),
                ('vads_card_brand', models.CharField(max_length=50, null=True, blank=True)),
                ('vads_card_number', models.CharField(max_length=16, null=True, blank=True)),
                ('vads_expiry_month', models.PositiveSmallIntegerField(null=True, blank=True)),
                ('vads_expiry_year', models.PositiveSmallIntegerField(null=True, blank=True)),
                ('vads_contract_used', models.CharField(max_length=250)),
                ('vads_auth_number', models.CharField(max_length=6, null=True, blank=True)),
                ('vads_auth_result', models.CharField(max_length=2, null=True, blank=True)),
                ('vads_auth_mode', models.CharField(blank=True, max_length=4, null=True, choices=[(b'FULL', b'FULL'), (b'MARK', b'MARK')])),
                ('vads_payment_certificate', models.CharField(max_length=40, null=True, blank=True)),
                ('vads_payment_src', models.CharField(blank=True, max_length=5, null=True, choices=[(b'EC', b'EC'), (b'MOTO', b'MOTO'), (b'CC', b'CC'), (b'OTHER', b'OTHER')])),
                ('vads_contrib', models.CharField(max_length=255, null=True, blank=True)),
                ('vads_user_info', models.CharField(max_length=255, null=True, blank=True)),
                ('vads_ext_trans_id', models.CharField(max_length=6, null=True, blank=True)),
                ('vads_payment_option_code', models.TextField(null=True, blank=True)),
                ('vads_change_rate', models.TextField(null=True, blank=True)),
            ],
            options={
                'verbose_name': 'Response',
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='ThemeConfig',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('success_footer_msg_return', models.TextField(null=True, blank=True)),
                ('cancel_footer_msg_return', models.TextField(null=True, blank=True)),
                ('secure_message', models.TextField(null=True, blank=True)),
                ('secure_message_register', models.TextField(null=True, blank=True)),
                ('site_id_label', models.TextField(null=True, blank=True)),
                ('css_for_payment', models.URLField(null=True, blank=True)),
                ('css_for_payment_mobile', models.URLField(null=True, blank=True)),
                ('header_for_mail', models.URLField(null=True, blank=True)),
                ('footer_for_mail', models.URLField(null=True, blank=True)),
                ('shop_logo', models.URLField(null=True, blank=True)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='VADContract',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('cb_contract_num', models.CharField(max_length=32)),
                ('amex_contract_num', models.CharField(max_length=32)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.AlterUniqueTogether(
            name='paymentresponse',
            unique_together=set([('vads_trans_id', 'vads_site_id', 'vads_trans_date')]),
        ),
        migrations.AddField(
            model_name='paymentrequest',
            name='theme',
            field=models.ForeignKey(blank=True, to='django_payzen.ThemeConfig', null=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='paymentrequest',
            name='user',
            field=models.ForeignKey(blank=True, to=settings.AUTH_USER_MODEL, null=True),
            preserve_default=True,
        ),
        migrations.AlterUniqueTogether(
            name='paymentrequest',
            unique_together=set([('vads_trans_id', 'vads_site_id', 'vads_trans_date')]),
        ),
    ]



import time

from django import template
from django.test import LiveServerTestCase

from .. import data
from ... import models

from selenium.webdriver.firefox import webdriver
from selenium.common import exceptions


class PayzenResponseTester(object):

    @classmethod
    def setUpClass(cls):
        cls.selenium = webdriver.WebDriver()
        super(PayzenResponseTester, cls).setUpClass()

    @classmethod
    def tearDownClass(cls):
        cls.selenium.quit()
        super(PayzenResponseTester, cls).tearDownClass()

    def generate_payment_form(self):
        t = template.Template(
            "{% load payzen_extras %}{% payzen_form instance %}")
        c = template.Context({
            "instance": self.instance
        })
        test = open("/tmp/payment_form_testing.html", "w")
        test.write(t.render(c))
        test.close()

    def validate_payment_form(self):
        self.selenium.get("file:///tmp/payment_form_testing.html")
        self.selenium.find_element_by_xpath('//input[@type="submit"]').click()

    def select_payment_card(self):
        self.selenium.find_element_by_id(
            'validationButtonPaymentMean').click()

    def enter_card_number(self):
        self.selenium.find_element_by_xpath(
            '//input[@name="vads_card_number"]').send_keys(
            self.card['card_number'])
        self.selenium.find_element_by_xpath(
            '//select[@name="vads_expiry_month"]/option[@value="12"]').click()
        self.selenium.find_element_by_xpath(
            '//select[@name="vads_expiry_year"]/option[@value="2024"]').click()
        self.selenium.find_element_by_xpath(
            '//input[@name="vads_cvv"]').send_keys("123")
        self.selenium.find_element_by_id("validationButtonCard").click()

    def response_object_tester(self):
        resp = models.PaymentResponse.objects.get(
            vads_trans_id=self.instance.vads_trans_id)
        for field_name, value in self.data.items():
            if hasattr(resp, field_name):
                self.assertEqual(str(getattr(resp, field_name)), value)
        if self.card['result'] == 'accepted':
            self.assertTrue(resp.payment_successful)
        else:
            self.assertFalse(resp.payment_successful)

    def test_payzen_response(self):
        self.assertFalse(
            self.instance.payment_successful)
        self.generate_payment_form()
        self.validate_payment_form()
        try:
            self.select_payment_card()
        except exceptions.NoSuchElementException:
            # For a recurring payment, there is no card selection step
            pass
        self.enter_card_number()
        time.sleep(10)  # Wait for confirmation request from Payzen
        self.response_object_tester()
        self.assertTrue(
            self.instance.payment_successful)


class BasicPaymentTest(PayzenResponseTester, LiveServerTestCase):

    def setUp(self):
        self.data = data.payment_args
        self.instance = models.PaymentRequest(
            **data.payment_args)
        self.instance.save()
        self.card = data.cards[0]


class CustomizedPaymentTest(PayzenResponseTester, LiveServerTestCase):

    def setUp(self):
        self.data = data.payment_args
        self.instance = models.PaymentRequest(
            **data.payment_args)
        self.instance.theme = models.ThemeConfig(
            **data.theme_args)
        self.instance.theme.save()
        self.instance.payment_config = models.MultiPaymentConfig(
            **data.payment_config_args)
        self.instance.payment_config.save()
        self.instance.save()
        self.css_class_to_find = "echeancier"
        self.card = data.cards[0]



import collections

url_exemple = "http://www.google.com/"

cards = [
    {
        'type': 'CB',
        'card_number': '4970100000000000',
        'behaviour': '3D-Secure',
        'result': 'accepted'
    },
    {
        'type': 'MasterCard',
        'card_number': '5970100300000000',
        'behaviour': '3D-Secure',
        'result': 'accepted'
    },
    {
        'type': 'Maestro',
        'card_number': '5000550000000000',
        'behaviour': '3D-Secure',
        'result': 'accepted'
    },
    {
        'type': 'CB',
        'card_number': '4970100000000009',
        'behaviour': '3D-Secure interactive',
        'result': 'accepted'
    },
    {
        'type': 'MasterCard',
        'card_number': '5970100300000009',
        'behaviour': '3D-Secure interactive',
        'result': 'accepted'
    },
    {
        'type': 'Maestro',
        'card_number': '5000550000000009',
        'behaviour': '3D-Secure interactive',
        'result': 'accepted'
    },
    {
        'type': 'CB',
        'card_number': '4970100000000003',
        'behaviour': 'Merchant without 3D-secure',
        'result': 'accepted'
    },
    {
        'type': 'MasterCard',
        'card_number': '5970100300000003',
        'behaviour': 'Merchant without 3D-secure',
        'result': 'accepted'
    },
    {
        'type': 'Maestro',
        'card_number': '5000550000000003',
        'behaviour': 'Merchant without 3D-secure',
        'result': 'accepted'
    },
    {
        'type': 'CB',
        'card_number': '4970100000000001',
        'behaviour': 'Buyer without 3D-secure',
        'result': 'accepted'
    },
    {
        'type': 'MasterCard',
        'card_number': '5970100300000001',
        'behaviour': 'Buyer without 3D-secure',
        'result': 'accepted'
    },
    {
        'type': 'Maestro',
        'card_number': '5000550000000001',
        'behaviour': 'Buyer without 3D-secure',
        'result': 'accepted'
    },
    {
        'type': 'CB',
        'card_number': '4970100000000002',
        'behaviour': 'Transaction to force',
        'result': 'rejected'
    },
    {
        'type': 'MasterCard',
        'card_number': '5970100300000002',
        'behaviour': 'Transaction to force',
        'result': 'rejected'
    },
    {
        'type': 'Maestro',
        'card_number': '5000550000000002',
        'behaviour': 'Transaction to force',
        'result': 'rejected'
    },
    {
        'type': 'CB',
        'card_number': '4970100000000007',
        'behaviour': 'Warranty = NO',
        'result': 'accepted'
    },
    {
        'type': 'MasterCard',
        'card_number': '5970100300023006',
        'behaviour': 'Warranty = NO',
        'result': 'accepted'
    },
    {
        'type': 'Maestro',
        'card_number': '5000550000023006',
        'behaviour': 'Warranty = NO',
        'result': 'accepted'
    },
    {
        'type': 'CB',
        'card_number': '4970100000000097',
        'behaviour': '3-D Secure authentication failed',
        'result': 'rejected'
    },
    {
        'type': 'MasterCard',
        'card_number': '5970100300000097',
        'behaviour': '3-D Secure authentication failed',
        'result': 'rejected'
    },
    {
        'type': 'Maestro',
        'card_number': '5000550000000097',
        'behaviour': '3-D Secure authentication failed',
        'result': 'rejected'
    },
    {
        'type': 'CB',
        'card_number': '4970100000000098',
        'behaviour': 'Card payment limit exceeded',
        'result': 'rejected'
    },
    {
        'type': 'MasterCard',
        'card_number': '5970100300000098',
        'behaviour': 'Card payment limit exceeded',
        'result': 'rejected'
    },
    {
        'type': 'Maestro',
        'card_number': '5000550000000098',
        'behaviour': 'Card payment limit exceeded',
        'result': 'rejected'
    },
    {
        'type': 'CB',
        'card_number': '4970100000000099',
        'behaviour': 'Wrong cryptogram',
        'result': 'rejected'
    },
    {
        'type': 'MasterCard',
        'card_number': '5970100300000099',
        'behaviour': 'Wrong cryptogram',
        'result': 'rejected'
    },
    {
        'type': 'Maestro',
        'card_number': '5000550000000099',
        'behaviour': 'Wrong cryptogram',
        'result': 'rejected'
    },
]

theme_args = collections.OrderedDict([
    ("success_footer_msg_return", "Success footer msg test"),
    ("cancel_footer_msg_return", "Cancel footer msg test"),
    ("secure_message", "Secure message test"),
    ("secure_message_register", "Secure message register test"),
    ("site_id_label", "Site ID label test"),
    ("css_for_payment", url_exemple+"payment.css"),
    ("css_for_payment_mobile", url_exemple+"mobile_payment.css"),
    ("header_for_mail", url_exemple+"mail_header.html"),
    ("footer_for_mail", url_exemple+"footer_mail.html"),
    ("shop_logo", url_exemple+"logo.png"),
])

payment_config_args = {
    "first": 5000,
    "count": 2,
    "period": 5
}

payment_args = {
    # Base fields
    "vads_amount": "10000",
    "vads_capture_delay": "2",
    "vads_payment_cards": "CB;Visa",
    "vads_return_mode": "NONE",
    "vads_validation_mode": "1",
    "vads_url_success": url_exemple,
    "vads_url_referral": url_exemple,
    "vads_url_refused": url_exemple,
    "vads_url_cancel": url_exemple,
    "vads_url_error": url_exemple,
    "vads_url_return": url_exemple,
    "vads_user_info": "Abbath Doom Occulta",
    "vads_shop_name": "Immortal",
    "vads_redirect_success_timeout": "1",
    "vads_redirect_success_message": "Tragedies Blows At Horizon",
    "vads_redirect_error_timeout": "1",
    "vads_redirect_error_message": "At The Heart Of Winter",
    # customer fields
    "vads_cust_address": "Oeschstr.",
    "vads_cust_address_number": "9",
    "vads_cust_country": "GE",
    "vads_cust_email": "test@nuclearblast.de",
    "vads_cust_id": "1",
    "vads_cust_name": "NUCLEAR BLAST",
    "vads_cust_cell_phone": "+49 7162 9280-0",
    "vads_cust_phone": "+49 7162 9280 26",
    "vads_cust_title": "Guitarist",
    "vads_cust_city": "Donzdorf",
    "vads_cust_state": "Donzdorf",
    "vads_cust_zip": "73072",
    "vads_language": "fr",
    # order fields
    "vads_order_id": "1234567890",
    "vads_order_info": "Order test info 1",
    "vads_order_info2": "Order test info 2",
    "vads_order_info3": "Order test info 3",
    # shipping fields
    "vads_ship_to_name": "NUCLEAR BLAST",
    "vads_ship_to_street_number": "9",
    "vads_ship_to_street": "Oeschstr. 9",
    "vads_ship_to_street2": "...",
    "vads_ship_to_zip": "73072",
    "vads_ship_to_city": "Donzdorf",
    "vads_ship_to_country": "GE",
    "vads_ship_to_phone_num": "+49 7162 9280-0",
    "vads_ship_to_state": "Donzdorf"
}



from django.contrib import admin

from . import models


class PaymentRequestAdmin(admin.ModelAdmin):
    model = models.PaymentRequest
    list_display = ("vads_trans_id", "vads_trans_date",
                    "vads_amount", "get_vads_currency_display")


class PaymentResponseAdmin(admin.ModelAdmin):
    model = models.PaymentResponse
    list_display = ("vads_trans_id", "vads_trans_date",
                    "vads_amount", "get_vads_currency_display",
                    "vads_operation_type")


admin.site.register(models.PaymentRequest, PaymentRequestAdmin)
admin.site.register(models.PaymentResponse, PaymentResponseAdmin)



from django import template
from django.test import LiveServerTestCase

from .. import data
from ... import models

from selenium.webdriver.firefox import webdriver


class PaymentInitTester(object):

    @classmethod
    def setUpClass(cls):
        cls.selenium = webdriver.WebDriver()
        super(PaymentInitTester, cls).setUpClass()

    @classmethod
    def tearDownClass(cls):
        cls.selenium.quit()
        super(PaymentInitTester, cls).tearDownClass()

    def generate_payment_form(self):
        t = template.Template(
            "{% load payzen_extras %}{% payzen_form instance %}")
        c = template.Context({
            "instance": self.instance
        })
        test = open("/tmp/payment_form_testing.html", "w")
        test.write(t.render(c))
        test.close()

    def payment_form_test(self):
        self.selenium.get("file:///tmp/payment_form_testing.html")
        self.selenium.find_element_by_xpath('//input[@type="submit"]').click()

    def payzen_form_test(self):
        self.assertTrue(
            self.selenium.current_url.startswith("https://secure.payzen.eu"))
        for css_class in self.css_classes_to_find:
            self.selenium.find_element_by_class_name(css_class)

    def test_payment_initialization(self):
        self.generate_payment_form()
        self.payment_form_test()
        self.payzen_form_test()


class BasicPaymentTest(PaymentInitTester, LiveServerTestCase):

    def setUp(self):
        self.instance = models.PaymentRequest(
            **data.payment_args)
        self.instance.save()
        self.css_classes_to_find = [
            "cardNumber", "vads-expiry-date-input", "validationButton", ]


class CustomizedPaymentTest(PaymentInitTester, LiveServerTestCase):

    def setUp(self):
        self.instance = models.PaymentRequest(
            **data.payment_args)
        self.instance.theme = models.ThemeConfig(
            **data.theme_args)
        self.instance.theme.save()
        self.instance.payment_config = models.MultiPaymentConfig(
            **data.payment_config_args)
        self.instance.payment_config.save()
        self.instance.save()
        self.css_classes_to_find = [
            "echeancier", "cardNumber", "vads-expiry-date-input",
            "validationButton", ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('django_payzen', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='paymentresponse',
            name='vads_action_mode',
            field=models.CharField(blank=True, max_length=11, null=True, choices=[(b'INTERACTIVE', b'INTERACTIVE'), (b'SILENT', b'SILENT')]),
        ),
        migrations.AlterField(
            model_name='paymentresponse',
            name='vads_contract_used',
            field=models.CharField(max_length=250, null=True, blank=True),
        ),
        migrations.AlterField(
            model_name='paymentresponse',
            name='vads_result',
            field=models.CharField(max_length=2, choices=[(b'00', b'Payment successful'), (b'02', b'Merchant should contact his bank'), (b'05', b'Payment refused'), (b'17', b'Payment cancelled by client'), (b'30', b'Wrong request format'), (b'96', b'Technical error during payment process')]),
        ),
        migrations.AlterField(
            model_name='paymentresponse',
            name='vads_url_check_src',
            field=models.CharField(max_length=10, choices=[(b'PAY', b'PAY'), (b'BO', b'BO'), (b'BATCH', b'BATCH'), (b'BATCH_AUTO', b'BATCH_AUTO'), (b'FILE', b'FILE'), (b'REC', b'REC'), (b'MERCH_BO', b'MERCH_BO')]),
        ),
    ]



from distutils.core import Command


class TestCommand(Command):
    description = "Launch all tests under django_payzen app"
    user_options = [
        ("site=", None, "Payzen site id (VADS_SITE_ID)"),
        ("certificate=", None, "Payzen certificate (VADS_CERTIFICATE)"),
    ]

    def initialize_options(self):
        self.site = None
        self.certificate = None

    def finalize_options(self):
        if self.site is None:
            raise Exception("\'site\' option is not provided.\n"
                            "Please call \'python setup.py test\' with"
                            " --site option")
        if self.certificate is None:
            raise Exception("\'certificate\' option is not provided.\n"
                            "Please call \'python setup.py test\' with"
                            " --certificate option")

    def configure_settings(self):
        from django.conf import settings
        settings.configure(
            DATABASES={
                "default": {
                    "NAME": ":memory:",
                    "ENGINE": "django.db.backends.sqlite3"
                }
            },
            INSTALLED_APPS=(
                "django.contrib.auth",
                "django.contrib.contenttypes",
                "django_payzen",
            ),
            MIDDLEWARE_CLASSES = (
                "django.contrib.sessions.middleware.SessionMiddleware",
                "django.middleware.common.CommonMiddleware",
                "django.middleware.csrf.CsrfViewMiddleware",
                "django.contrib.auth.middleware.AuthenticationMiddleware",
                "django.contrib.messages.middleware.MessageMiddleware",
            ),
            VADS_SITE_ID = self.site,
            VADS_CERTIFICATE = self.certificate,
            VADS_CURRENCY = "978",
            VADS_ACTION_MODE = "INTERACTIVE",
            VADS_CTX_MODE = "TEST",
            LOGGING = {
                "version": 1,
                "disable_existing_loggers": True,
                "handlers": {
                    "console": {
                        "level": "INFO",
                        "class": "logging.StreamHandler",
                    },
                },
                "loggers": {
                    "django_payzen": {
                        "handlers": ["console"],
                        "propagate": True,
                        "level": "INFO"
                    }
                },
            },
            STATIC_URL = "/static/",
            MEDIA_URL = "/media/",
            STATIC_ROOT = "/tmp/static/",
            MEDIA_ROOT = "/tmp/media/",
            ROOT_URLCONF = "tests.urls",
        )

    def run(self):
        import django
        from django.core.management import call_command
        self.configure_settings()
        django.setup()
        call_command("test", "django_payzen", liveserver="0.0.0.0:8000")



"""
Models for Payzen - Payment using webform v2

The models are implemented according to the document :
'Guide_d_implementation_formulaire_paiement_V2_Payzen_v2.9f.pdf'

TODO add validators to manage payzen contraints on special fields.
"""

import datetime

from django.conf import settings
from django.db import models
from django.utils.encoding import python_2_unicode_compatible
from django.utils.timezone import utc

from . import app_settings
from . import tools
from . import constants


auth_user_model = getattr(settings, 'AUTH_USER_MODEL', 'auth.User')


@python_2_unicode_compatible
class ThemeConfig(models.Model):
    """Model that contains the theme parameters of the payment form.

    To set up a theme is optional.
    """
    success_footer_msg_return = models.TextField(blank=True, null=True)
    cancel_footer_msg_return = models.TextField(blank=True, null=True)
    secure_message = models.TextField(blank=True, null=True)
    secure_message_register = models.TextField(blank=True, null=True)
    site_id_label = models.TextField(blank=True, null=True)
    css_for_payment = models.URLField(blank=True, null=True)
    css_for_payment_mobile = models.URLField(blank=True, null=True)
    header_for_mail = models.URLField(blank=True, null=True)
    footer_for_mail = models.URLField(blank=True, null=True)
    shop_logo = models.URLField(blank=True, null=True)

    def __str__(self):
        vads_theme_config = ""
        for field in self._meta.fields:
            if not field.auto_created:
                vads_theme_config += "{}={};".format(
                    field.name.upper(), field.value_from_object(self))
        return vads_theme_config[:-1]


@python_2_unicode_compatible
class MultiPaymentConfig(models.Model):
    """Model that contains parameters for multi-payment.

    Model fields:
    first -- the amount of the first payment.
    count -- the number of payments.
    period -- the number of days between two payments.

    Exemple:
    If you setup vads_amount to 10000 and a MultiPaymentConfig with
    first=5000, count=3, period=2, a first payment of 5000  will take place
    in 2 days from now, then a 2nd payment of 2500 will append 2 days after
    the first payment and a third payment of 2500 2 days later.
    """
    first = models.PositiveIntegerField()
    count = models.PositiveIntegerField()
    period = models.PositiveIntegerField()

    def __str__(self):
        return "MULTI:first={};count={};period={}".format(
            self.first, self.count, self.period)


@python_2_unicode_compatible
class CustomPaymentConfig(models.Model):
    """Model to manage a custom payment scheduler."""
    date = models.DateField()
    amount = models.PositiveIntegerField()

    def __str__(self):
        return "{}={}".format(
            self.date.strftime("%Y%m%d"), self.amount)


@python_2_unicode_compatible
class VADContract(models.Model):
    """Model to manage several VAD contrats.

    This model is useful when you have several VAD contracts and
    you want to dynamically change the VAD contract used."""
    cb_contract_num = models.CharField(max_length=32)
    amex_contract_num = models.CharField(max_length=32)

    def __str__(self):
        return "vads_contracts=CB={};AMEX={}".format(
            self.cb_contract_num, self.amex_contract_num)


class CustomerDetails(models.Model):
    """Abstract model that contains all data relative to the customer."""

    vads_cust_address = models.CharField(
        max_length=255, blank=True, null=True)
    vads_cust_address_number = models.CharField(
        max_length=5, blank=True, null=True)
    vads_cust_country = models.CharField(
        max_length=2, blank=True, null=True)
    vads_cust_email = models.EmailField(
        blank=True, null=True)
    vads_cust_id = models.CharField(
        max_length=63, blank=True, null=True)
    vads_cust_name = models.CharField(
        max_length=127, blank=True, null=True)
    vads_cust_last_name = models.CharField(
        max_length=63, blank=True, null=True)
    vads_cust_first_name = models.CharField(
        max_length=63, blank=True, null=True)
    vads_cust_cell_phone = models.CharField(
        max_length=32, blank=True, null=True)
    vads_cust_phone = models.CharField(
        max_length=32, blank=True, null=True)
    vads_cust_title = models.CharField(
        max_length=63, blank=True, null=True)
    vads_cust_city = models.CharField(
        max_length=63, blank=True, null=True)
    vads_cust_status = models.CharField(
        max_length=63, blank=True, null=True)
    vads_cust_state = models.CharField(
        max_length=63, blank=True, null=True)
    vads_cust_zip = models.CharField(
        max_length=63, blank=True, null=True)
    vads_language = models.CharField(
        max_length=2, blank=True, null=True)

    class Meta:
        abstract = True


class OrderDetails(models.Model):
    """Abstract model that contains all data relative to the order."""

    vads_order_id = models.CharField(
        max_length=32, blank=True, null=True)
    vads_order_info = models.CharField(
        max_length=255, blank=True, null=True)
    vads_order_info2 = models.CharField(
        max_length=255, blank=True, null=True)
    vads_order_info3 = models.CharField(
        max_length=255, blank=True, null=True)

    class Meta:
        abstract = True


class ShippingDetails(models.Model):
    """Abstract model that contains all data relative to shipping."""

    vads_ship_to_name = models.CharField(
        max_length=127, blank=True, null=True)
    vads_ship_to_first_name = models.CharField(
        max_length=63, blank=True, null=True)
    vads_ship_to_last_name = models.CharField(
        max_length=63, blank=True, null=True)
    vads_ship_to_street_number = models.CharField(
        max_length=5, blank=True, null=True)
    vads_ship_to_street = models.CharField(
        max_length=255, blank=True, null=True)
    vads_ship_to_street2 = models.CharField(
        max_length=255, blank=True, null=True)
    vads_ship_to_zip = models.CharField(
        max_length=63, blank=True, null=True)
    vads_ship_to_city = models.CharField(
        max_length=63, blank=True, null=True)
    vads_ship_to_country = models.CharField(
        max_length=2, blank=True, null=True)
    vads_ship_to_phone_num = models.CharField(
        max_length=32, blank=True, null=True)
    vads_ship_to_state = models.CharField(
        max_length=255, blank=True, null=True)

    class Meta:
        abstract = True


class RequestDetails(models.Model):
    """Abstract model that contains all data relative to transaction."""

    vads_action_mode = models.CharField(
        max_length=11, choices=constants.VADS_ACTION_MODE_CHOICES,
        default=app_settings.VADS_ACTION_MODE)
    vads_amount = models.PositiveIntegerField()
    vads_currency = models.CharField(
        max_length=3, choices=constants.VADS_CURRENCY_CHOICES,
        default=app_settings.VADS_CURRENCY)
    vads_ctx_mode = models.CharField(
        max_length=10, choices=constants.VADS_CTX_MODE_CHOICES,
        default=app_settings.VADS_CTX_MODE)
    vads_page_action = models.CharField(
        max_length=7, default='PAYMENT')
    vads_payment_config = models.TextField()
    vads_site_id = models.PositiveIntegerField(
        default=app_settings.VADS_SITE_ID)
    vads_trans_date = models.CharField(
        max_length=14)
    vads_trans_id = models.CharField(max_length=6)
    vads_version = models.CharField(
        max_length=2, default='V2')
    signature = models.CharField(
        max_length=40)

    class Meta:
        abstract = True


class WarrantyDetails(models.Model):

    vads_threeds_enrolled = models.CharField(
        max_length=1, choices=constants.VADS_THREEDS_ENROLLED,
        blank=True, null=True)
    vads_threeds_cavv = models.CharField(max_length=28, blank=True, null=True)
    vads_threeds_eci = models.CharField(max_length=2, blank=True, null=True)
    vads_threeds_xid = models.CharField(max_length=28, blank=True, null=True)
    vads_threeds_cavvAlgorithm = models.CharField(
        choices=constants.VADS_THREEDS_CAVVALGORITHM_CHOICES,
        max_length=1, blank=True, null=True)
    vads_threeds_status = models.CharField(
        choices=constants.VADS_THREEDS_STATUS_CHOICES,
        max_length=1, blank=True, null=True)
    vads_threeds_sign_valid = models.NullBooleanField()
    vads_threeds_error_code = models.CharField(
        max_length=2, blank=True, null=True)
    vads_threeds_exit_status = models.CharField(
        max_length=2, blank=True, null=True)
    vads_warranty_result = models.CharField(
        max_length=13, blank=True, null=True)

    class Meta:
        abstract = True


class RiskControlDetail(models.Model):

    vads_extra_result = models.CharField(max_length=2, blank=True, null=True)
    vads_card_country = models.CharField(max_length=2, blank=True, null=True)
    vads_bank_code = models.CharField(max_length=2, blank=True, null=True)
    vads_bank_product = models.CharField(max_length=3, blank=True, null=True)

    class Meta:
        abstract = True


class CustomizationDetails(models.Model):

    vads_available_languages = models.TextField(blank=True, null=True)
    vads_theme_config = models.CharField(max_length=255, blank=True, null=True)
    vads_shop_url = models.URLField(blank=True, null=True)
    vads_shop_name = models.CharField(max_length=255, blank=True, null=True)

    class Meta:
        abstract = True


class PaymentResponse(WarrantyDetails, CustomerDetails,
                      OrderDetails, ShippingDetails,
                      CustomizationDetails):

    """Model that contains the main response parameters from Payzen."""

    signature = models.CharField(max_length=40)
    vads_ctx_mode = models.CharField(
        max_length=10, choices=constants.VADS_CTX_MODE_CHOICES)
    vads_url_check_src = models.CharField(
        choices=constants.VADS_URL_CHECK_SRC_CHOICES,
        max_length=10, blank=True, null=True)
    vads_version = models.CharField(max_length=2)
    vads_trans_date = models.CharField(max_length=14)
    vads_action_mode = models.CharField(
        max_length=11, choices=constants.VADS_ACTION_MODE_CHOICES,
        blank=True, null=True)

    vads_trans_id = models.CharField(max_length=6)
    vads_payment_config = models.TextField(
        blank=True, null=True)
    vads_sequence_number = models.PositiveSmallIntegerField(
        blank=True, null=True)
    vads_site_id = models.PositiveIntegerField()
    vads_amount = models.PositiveIntegerField(
        blank=True, null=True)
    vads_currency = models.CharField(
        max_length=3, choices=constants.VADS_CURRENCY_CHOICES,
        blank=True, null=True)
    vads_effective_amount = models.PositiveIntegerField(
        blank=True, null=True)
    vads_operation_type = models.CharField(
        max_length=6, choices=constants.VADS_OPERATION_TYPE_CHOICES,
        blank=True, null=True)
    vads_result = models.CharField(
        max_length=2, choices=constants.VADS_RESULT_CHOICES)
    vads_validation_mode = models.CharField(
        choices=constants.VADS_VALIDATION_MODE_CHOICES,
        max_length=1, blank=True, null=True)
    vads_trans_status = models.CharField(
        choices=constants.VADS_TRANS_STATUS, max_length=33)
    vads_effective_creation_date = models.CharField(
        max_length=14, blank=True, null=True)
    vads_presentation_date = models.CharField(
        max_length=14, blank=True, null=True)
    vads_capture_delay = models.PositiveIntegerField(blank=True, null=True)
    vads_card_brand = models.CharField(max_length=50, blank=True, null=True)
    vads_card_number = models.CharField(max_length=16, blank=True, null=True)
    vads_expiry_month = models.PositiveSmallIntegerField(blank=True, null=True)
    vads_expiry_year = models.PositiveSmallIntegerField(blank=True, null=True)
    vads_contract_used = models.CharField(
        max_length=250, blank=True, null=True)
    vads_auth_number = models.CharField(max_length=6, blank=True, null=True)
    vads_auth_result = models.CharField(max_length=2, blank=True, null=True)
    vads_auth_mode = models.CharField(
        max_length=4, choices=constants.VADS_AUTH_MODE_CHOICES,
        blank=True, null=True)
    vads_payment_certificate = models.CharField(
        max_length=40, blank=True, null=True)
    vads_payment_src = models.CharField(
        max_length=5, blank=True, null=True,
        choices=constants.VADS_PAYMENT_SRC_CHOICES)
    vads_contrib = models.CharField(
        max_length=255, blank=True, null=True)
    vads_user_info = models.CharField(max_length=255, blank=True, null=True)
    vads_ext_trans_id = models.CharField(
        max_length=6, blank=True, null=True)
    vads_payment_option_code = models.TextField(blank=True, null=True)

    vads_change_rate = models.TextField(blank=True, null=True)

    class Meta:
        verbose_name = "Response"
        unique_together = ("vads_trans_id", "vads_site_id", "vads_trans_date")

    @property
    def payment_successful(self):
        if self.vads_result == '00':
            return True
        return False

    @property
    def request(self):
        return PaymentRequest.objects.get(
            vads_trans_id=self.vads_trans_id,
            vads_trans_date=self.vads_trans_date,
            vads_site_id=self.vads_site_id)

    @property
    def user(self):
        try:
            return self.request.user
        except AttributeError:
            return None


class PaymentRequest(RequestDetails, CustomerDetails,
                     OrderDetails, ShippingDetails):
    """Model that contains all Payzen parameters to initiate a payment."""

    user = models.ForeignKey(auth_user_model, blank=True, null=True)

    vads_capture_delay = models.PositiveIntegerField(blank=True, null=True)
    vads_contrib = models.CharField(
        max_length=255, blank=True, null=True,
        default=app_settings.VADS_CONTRIB)
    vads_payment_cards = models.CharField(
        max_length=127, blank=True, null=True)
    vads_return_mode = models.CharField(
        max_length=12, choices=constants.VADS_RETURN_MODE_CHOICES,
        blank=True, null=True)
    vads_theme_config = models.CharField(
        max_length=255, blank=True, null=True)
    vads_validation_mode = models.CharField(
        choices=constants.VADS_VALIDATION_MODE_CHOICES,
        max_length=1, blank=True, null=True)
    vads_url_success = models.URLField(blank=True, null=True)
    vads_url_referral = models.URLField(blank=True, null=True)
    vads_url_refused = models.URLField(blank=True, null=True)
    vads_url_cancel = models.URLField(blank=True, null=True)
    vads_url_error = models.URLField(blank=True, null=True)
    vads_url_return = models.URLField(blank=True, null=True)
    vads_user_info = models.CharField(max_length=255, blank=True, null=True)
    vads_shop_name = models.CharField(max_length=255, blank=True, null=True)
    vads_redirect_success_timeout = models.PositiveIntegerField(
        blank=True, null=True)
    vads_redirect_success_message = models.CharField(
        max_length=255, blank=True, null=True)
    vads_redirect_error_timeout = models.PositiveIntegerField(
        blank=True, null=True)
    vads_redirect_error_message = models.CharField(
        max_length=255, blank=True, null=True)

    # Relations
    theme = models.ForeignKey(ThemeConfig, blank=True, null=True)
    payment_config = models.ForeignKey(
        MultiPaymentConfig, blank=True, null=True)
    custom_payment_config = models.ManyToManyField(CustomPaymentConfig)

    class Meta:
        verbose_name = "Request"
        unique_together = ("vads_trans_id", "vads_site_id", "vads_trans_date")

    def set_vads_payment_config(self):
        """
        vads_payment_config can be set only after object saving.

        A custom payment config can be set once PaymentRequest saved
        (adding elements to the m2m relationship). As a consequence
        we set vads_payment_config just before sending data elements
        to payzen."""
        self.vads_payment_config = tools.get_vads_payment_config(
            self.payment_config, self.custom_payment_config.all())

    def set_signature(self):
        self.signature = tools.get_signature(self)

    @property
    def response(self):
        try:
            return PaymentResponse.objects.get(
                vads_trans_id=self.vads_trans_id,
                vads_trans_date=self.vads_trans_date,
                vads_site_id=self.vads_site_id)
        except PaymentResponse.DoesNotExist:
            return PaymentResponse.objects.none()

    @property
    def payment_successful(self):
        return self.response and self.response.payment_successful

    def save(self):
        """
        We set up vads_trans_id and theme according to payzen format.

        If fields values are explicitely set by user, we do not override
        their values.
        """
        if not self.vads_trans_date:
            self.vads_trans_date = datetime.datetime.utcnow().replace(
                tzinfo=utc).strftime("%Y%m%d%H%M%S")
        if not self.vads_trans_id:
            self.vads_trans_id = tools.get_vads_trans_id(
                self.vads_site_id, self.vads_trans_date)
        if self.theme and not self.vads_theme_config:
            self.vads_theme_config = str(self.theme)
        if not self.pk:
            super(PaymentRequest, self).save()
        self.set_vads_payment_config()
        self.set_signature()
        super(PaymentRequest, self).save()

    def update(self):
        if not self.pk:
            # Prevent bug on filtering m2m relationship
            self.save()
        if not self.vads_trans_date:
            self.vads_trans_date = datetime.datetime.utcnow().replace(
                tzinfo=utc).strftime("%Y%m%d%H%M%S")
        self.set_vads_payment_config()
        self.set_signature()
        self.save()



"""Setup file for easy installation"""
import os

from setuptools import setup, find_packages

from tests import test_cmd


ROOT = os.path.dirname(__file__)
PIP_REQUIRES = os.path.join(ROOT, "requirements.txt")


setup(
    name="django-payzen",
    version="1.0.1",
    description="Django app to manage payments with Payzen ETP",
    license='MIT',
    author="Bertrand Svetchine",
    author_email="bertrand.svetchine@gmail.com",
    url="https://github.com/bsvetchine/django-payzen",
    packages=find_packages(),
    include_package_data=True,
    install_requires=["Django"],
    classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: Web Environment",
        "Intended Audience :: Developers",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.2",
        "Programming Language :: Python :: 3.3",
        "Programming Language :: Python :: 3.4",
        "Framework :: Django",
        "Topic :: Software Development"],
    cmdclass={'test': test_cmd.TestCommand}
)



import hashlib
import random

from . import app_settings


def get_vads_trans_id(vads_site_id, vads_trans_date):
    """
    Returns a default value for vads_trans_id field.

    vads_trans_id field is mandatory. It is composed by 6 numeric
    characters that identifies the transaction. There is a unicity contraint
    between vads_site_id and vads_trans_date (the first 8 characters
    representing the transaction date).

    We consider the probability of having 2 identical
    generated vads_trans_id in the same day as null."""
    vads_trans_id = ""
    for i in range(0, 6):
        vads_trans_id += str(random.randint(0, 9))
    return vads_trans_id


def get_custom_payment_config(payments_conf):
    """
    Returns the payzen value for vads_payment_config for multi payments.

    If CustomPaymentConfig are set (m2m relationship), returns the payzen
    string for vads_payment_config.
    """
    conf = 'MULTI_EXT:'
    for pc in payments_conf:
        conf += str(pc) + ';'
    return conf[:-1]


def get_vads_payment_config(general_conf, custom_payments_conf):
    """
    Returns the vads_payment_config string respecting to payzen format.

    According to the foreignKey relationship to MultiPaymentConfig or the
    ManyToMany relationship to CustomPaymentConfig, returns the value
    for vads_payment_config.
    It should not be possible to edit at the same time a MultiPaymentConfig
    and a CustomPaymentConfig (does not make sense). It this case appends,
    the CustomPaymentConfig only is considered.
    """
    if custom_payments_conf:
        return get_custom_payment_config(custom_payments_conf)
    if general_conf:
        return str(general_conf)
    return 'SINGLE'


def get_signature(payment_request):
    """
    Returns the signature for the transaction.

    To compute the signature, first you have to get the value of all
    the fields that starts by 'vads_', ordering them alphabetically.
    All the values are separated by the '+' character. Then you add
    the value of the payzen certificate.
    Finaly you hash the string using sha1."""

    vads_args = {}
    for field in payment_request._meta.fields:
        if field.name[:5] == 'vads_':
            field_value = field.value_from_object(payment_request)
            if field_value:
                vads_args.update({
                    field.name: field_value
                })
    base_str = ''
    for key in sorted(vads_args):
        base_str += str(vads_args[key]) + '+'
    base_str += app_settings.VADS_CERTIFICATE
    return hashlib.sha1(base_str.encode("utf-8")).hexdigest()


def is_signature_valid(post_args):
    vads_args = [arg for arg in post_args if arg.startswith("vads_")]
    signature_str = ""
    for key in sorted(vads_args):
        signature_str += post_args[key] + "+"
    signature_str += app_settings.VADS_CERTIFICATE
    return post_args.get("signature") and (hashlib.sha1(
        signature_str.encode("utf-8")).hexdigest() == post_args["signature"])



from django import forms

from . import models


class PaymentRequestForm(forms.ModelForm):

    class Meta:
        model = models.PaymentRequest
        exclude = []

    def __remove_empty_fields(self):
        form_fields_to_remove = []
        for model_field_name, form_field in self.fields.items():
            if not getattr(self.instance, model_field_name):
                form_fields_to_remove.append(model_field_name)
        for model_field_name in form_fields_to_remove:
            self.fields.pop(model_field_name)

    def __set_all_fields_hidden(self):
        for field in self.fields.values():
            field.widget = forms.HiddenInput()

    def __init__(self, *args, **kwargs):
        super(PaymentRequestForm, self).__init__(*args, **kwargs)
        self.__remove_empty_fields()
        self.__set_all_fields_hidden()


class PaymentResponseForm(forms.ModelForm):

    class Meta:
        model = models.PaymentResponse
        exclude = []



"""django-payzen signals."""
import django.dispatch

response_error = django.dispatch.Signal()
payment_success = django.dispatch.Signal(
    providing_args=["response", "session_args"])
payment_failure = django.dispatch.Signal(
    providing_args=["response", "session_args"])



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('django_payzen', '0002_auto_20141024_0822'),
    ]

    operations = [
        migrations.AlterField(
            model_name='paymentresponse',
            name='vads_url_check_src',
            field=models.CharField(blank=True, max_length=10, null=True, choices=[(b'PAY', b'PAY'), (b'BO', b'BO'), (b'BATCH', b'BATCH'), (b'BATCH_AUTO', b'BATCH_AUTO'), (b'FILE', b'FILE'), (b'REC', b'REC'), (b'MERCH_BO', b'MERCH_BO')]),
            preserve_default=True,
        ),
    ]
